
module MySQLNoIo
  # A WrapStack collects a series of Wraps, and runs their #execute commands
  # inside one another.
  #
  # Example:
  #  stack = MySQLNoIo::WrapStack.new()a
  #
  #  wrapFoo = Wrap::Foo.new()
  #  wrapBar = Wrap::Bar.new()
  #
  #  stack.add_stack wrapFoo
  #  stack.add_stack wrapBar
  #  stack.execute do
  #    puts "hi"
  #  end
  #
  #  Is identical to:
  #  wrapFoo.execute do
  #    wrapBar.execute do
  #      puts "hi"
  #    end
  #  end
  #
  class WrapStack
    def initialize
      @wraps = []
    end

    # Add a layer to wrap around the execution of the command
    #
    # @param [MySQLNoIo::Wrap] the wrapper to add to the stack
    # @raise [MySQLNoIo::Errors::ArgumentError] if the wrapper is not sufficient
    #
    # @return [TrueClass]
    def add_wrap wrap
      unless wrap.respond_to?('execute')
        raise MySQLNoIo::Errors::ArgumentError.new('Wrap must respond to #execute and accept a block.')
      end

      @wraps << wrap

      true
    end

    # Execute your bit of code from within the safety and comfort of the
    # wrappers appended to the stack.
    def execute &block
      # Brace yourself.
      #
      # This next bit of code creates a nested lamda like the WrapStack
      # describes.
      deep_proc = @wraps.reverse.inject(block) do |deep_proc, layer|
        Proc.new do
          layer.execute do
            deep_proc.call
          end
        end
      end

      deep_proc.call
    end
  end
end

